#!/usr/bin/env fades

import subprocess
import webbrowser

import click  # fades


GH_MERGE_URL = (
    "https://github.com/{organization}/{project}/compare/{branch}?expand=1")


def get_output(cmd):
    """Run and return the output."""
    proc = subprocess.run(cmd, stdout=subprocess.PIPE, encoding='utf8')
    proc.check_returncode()
    return proc.stdout


def run(cmd):
    """Just run."""
    subprocess.run(cmd).check_returncode()


def _get_current_branch():
    cmd = ["git", "rev-parse", "--abbrev-ref", "HEAD"]
    output = get_output(cmd)
    return output.strip()


def _get_project_data():
    cmd = ["git", "remote", "get-url", "origin"]
    output = get_output(cmd).strip()
    p1, p2 = output.split(':')
    organization, project = p2.split('/')
    assert project.endswith(".git"), repr(project)
    project = project[:-4]
    return organization, project


def get_remotes(verbose=False):
    """Get all remote stuff."""
    cmd = ['git', 'remote']
    if verbose:
        cmd.append('-v')
    output = get_output(cmd)
    return output.split("\n")


def _get_branches():
    """Get all local branches."""
    result = get_output(['git', 'branch'])
    names = []
    for item in result.split('\n'):
        item = item.strip()
        if item:
            names.append(item.split()[1] if ' ' in item else item)
    return names


@click.command()
@click.argument(
    'branch',
    # help="What to review, 'branch' for own stuff, 'user:branch' for branch from others")
)
def review(branch):
    """Prepare everything for a review."""
    # FIXME: sync?

    branch_name = 'review_' + branch.replace(':', '_')
    current_branches = _get_branches()
    if branch_name in current_branches:
        run(['git', 'branch', '-D', branch_name])

    run(['git', 'checkout', '-b', branch_name])

    if ':' in branch:
        user, real_branch = branch.split(':')
        remotes = get_remotes(verbose=True)
        project = remotes[0].split()[1].split('/')[-1]
        gh_address = "https://github.com/{}/{}".format(user, project)
        run(['git', 'pull', gh_address, real_branch])
    else:
        run(['git', 'merge', 'origin/' + branch])


@click.command()
def push():
    """Push the current branch to Github and open the merging URL."""
    branch = _get_current_branch()
    run(['git', 'push', '--set-upstream', 'origin', branch])

    organization, project = _get_project_data()
    branch_url = GH_MERGE_URL.format(organization=organization, project=project, branch=branch)
    print("Opening", branch_url)
    webbrowser.open(branch_url)


@click.command()
def sync():
    """Sync local principal branch with remote one."""
    branch = _get_current_branch()
    subprocess.run(['git', 'fetch', 'upstream'])
    subprocess.run(['git', 'merge', 'upstream/{}'.format(branch)])


@click.command()
def branches():
    """Show all branches in a particular format and order."""
    cmd = [
        'git', 'for-each-ref', '--sort=committerdate', 'refs/heads/', (
            "--format=%(HEAD) %(refname:short) "
            "(%(color:green)%(committerdate:relative)%(color:reset))")]
    run(cmd)


@click.group()
def cli():
    pass


cli.add_command(sync)
cli.add_command(push)
cli.add_command(review)
cli.add_command(branches)
cli()
